/**
 * Copyright 2013 Bull S.A.S.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.commands.builtin.internal.completer;

import jline.console.completer.Completer;
import jline.console.completer.StringsCompleter;
import org.apache.felix.ipojo.annotations.Bind;
import org.apache.felix.ipojo.annotations.Component;
import org.apache.felix.ipojo.annotations.Provides;
import org.ow2.shelbie.core.registry.CommandRegistry;
import org.ow2.shelbie.core.registry.info.CommandInfo;

import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * This Completer provides completion support for command names.
 */
@Component(propagation = true)
@Provides(specifications = Completer.class)
public class CommandNameCompleter extends StringsCompleter {

    private CommandRegistry registry;

    public CommandNameCompleter() {
        super("");
    }

    @Override
    public int complete(String buffer, int cursor, List<CharSequence> clist) {
        // Update candidates
        getStrings().clear();
        getStrings().addAll(getCommands());
        return super.complete(buffer, cursor, clist);
    }

    private SortedSet<String> getCommands() {
        SortedSet<String> names = new TreeSet<String>();

        for (CommandInfo info : registry.getAllCommands()) {
            names.add(info.getScope() + ":" + info.getName());
        }

        return names;
    }

    @Bind
    public void bindRegistry(CommandRegistry registry) {
        this.registry = registry;
    }
}