/**
 * Copyright 2013 Peergreen S.A.S.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.commands.builtin.internal;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

import org.ow2.shelbie.testing.ActionContainer;
import org.testng.annotations.Test;

/**
 * User: guillaume
 * Date: 29/07/13
 * Time: 19:38
 */
public class GrepActionTestCase {

    @Test
    public void testReturnedLinesHaveTheResetSequenceAndNewLineAtTheEnd() throws Exception {
        ActionContainer container = new ActionContainer(new GrepAction());
        container.setSystemIn("guillaume");
        container.argument(0, "guillaume");
        container.execute(null);

        assertTrue(container.getSystemOut().endsWith("\u001B[m\n"));
    }

    @Test
    public void testResetEscapePatternIsFound() throws Exception {
        ActionContainer container = new ActionContainer(new GrepAction());
        container.setSystemIn("\u001B[1;31mHello\u001B[m wonderful world");
        container.argument(0, "wonderful");
        container.execute(null);

        assertEquals(container.getSystemOut(), "\u001B[1;31mHello\u001B[m \u001B[43;30mwonderful\u001B[m world\u001B[m\n");
    }

    @Test
    public void testMultiValuedEscapePatternIsFound() throws Exception {
        ActionContainer container = new ActionContainer(new GrepAction());
        container.setSystemIn("\u001B[1;31mHello wonderful world\u001B[m");
        container.argument(0, "wonderful");
        container.execute(null);

        assertEquals(container.getSystemOut(), "\u001B[1;31mHello \u001B[43;30mwonderful\u001B[1;31m world\u001B[m\u001B[m\n");
    }

    @Test
    public void testMonoValuedEscapePatternIsFound() throws Exception {
        ActionContainer container = new ActionContainer(new GrepAction());
        container.setSystemIn("\u001B[1mHello wonderful world\u001B[m");
        container.argument(0, "wonderful");
        container.execute(null);

        assertEquals(container.getSystemOut(), "\u001B[1mHello \u001B[43;30mwonderful\u001B[1m world\u001B[m\u001B[m\n");
    }
}
