/**
 * Copyright 2012 Bull S.A.S.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.handler;

import org.mockito.Mock;
import org.osgi.framework.ServiceReference;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.initMocks;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * A {@code CompleterServiceComparatorTestCase} is ...
 *
 * @author Guillaume Sauthier
 */
public class CompleterServiceComparatorTestCase {

    private CompleterServiceComparator comparator;

    @Mock
    private ServiceReference zero;

    @Mock
    private ServiceReference one;

    @Mock
    private ServiceReference two;


    @BeforeMethod
    public void setUp() throws Exception {
        initMocks(this);
        comparator = new CompleterServiceComparator();

        when(zero.getProperty(CompleterServiceComparator.POSITION)).thenReturn("0");
        when(one.getProperty(CompleterServiceComparator.POSITION)).thenReturn("1");
        when(two.getProperty(CompleterServiceComparator.POSITION)).thenReturn("2");

    }

    @Test
    public void testZeroOneComparison() throws Exception {
        assertTrue(comparator.compare(zero, one) < 0); // zero lesser than one
        assertTrue(comparator.compare(one, zero) > 0); // one bigger than zero
    }

    @Test
    public void testZeroTwoComparison() throws Exception {
        assertTrue(comparator.compare(zero, two) < 0); // zero lesser than two
        assertTrue(comparator.compare(two, zero) > 0); // two bigger than zero
    }

    @Test
    public void testOneTwoComparison() throws Exception {
        assertTrue(comparator.compare(one, two) < 0); // one lesser than two
        assertTrue(comparator.compare(two, one) > 0); // two bigger than one
    }

    @Test
    public void testEqualityComparison() throws Exception {
        assertEquals(comparator.compare(zero, zero), 0); // zero equals to zero
        assertEquals(comparator.compare(one, one), 0); // one equals to one
        assertEquals(comparator.compare(two, two), 0); // two equals to two
    }
}
