/**
 * Copyright 2013 Bull S.A.S.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.handler.completer;

import jline.TerminalSupport;
import jline.console.ConsoleReader;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * User: guillaume
 * Date: 17/02/13
 * Time: 17:30
 */
public class ShelbieCandidateListCompletionHandlerTestCase {
    private ConsoleReader reader;

    @BeforeMethod
    public void setUp() throws Exception {
        reader = new ConsoleReader(null, new ByteArrayOutputStream(), new TerminalSupport(true) { });
    }

    @Test
    public void testCommandCompletion1() throws Exception {
        ShelbieCandidateListCompletionHandler handler = new ShelbieCandidateListCompletionHandler();

        reader.getCursorBuffer().write("in");

        List<CharSequence> candidates = new ArrayList<CharSequence>();
        candidates.add("felix:install");
        candidates.add("shelbie:insert");
        //candidates.add("inhibit");
        assertTrue(handler.complete(reader, candidates, 0));
        assertEquals(reader.getCursorBuffer().toString(), "ins");

    }

    @Test
    public void testCommandCompletion2() throws Exception {
        ShelbieCandidateListCompletionHandler handler = new ShelbieCandidateListCompletionHandler();

        reader.getCursorBuffer().write("help ");

        List<CharSequence> candidates = new ArrayList<CharSequence>();
        candidates.add("felix:ls");
        candidates.add("shelbie:grep");
        assertTrue(handler.complete(reader, candidates, 5));
        assertEquals(reader.getCursorBuffer().toString(), "help ");
    }

    @Test
    public void testCommandCompletion3() throws Exception {
        ShelbieCandidateListCompletionHandler handler = new ShelbieCandidateListCompletionHandler();

        reader.getCursorBuffer().write("fac");

        List<CharSequence> candidates = new ArrayList<CharSequence>();
        candidates.add("factories");
        candidates.add("factory");
        assertTrue(handler.complete(reader, candidates, 0));
        assertEquals(reader.getCursorBuffer().toString(), "factor");
    }
}
