/**
 * Copyright 2013 Peergreen SAS
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.history;

import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import javax.security.auth.Subject;
import java.security.Principal;
import java.security.acl.Group;

import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

/**
 * Created with IntelliJ IDEA.
 * User: guillaume
 * Date: 07/02/13
 * Time: 17:40
 * To change this template use File | Settings | File Templates.
 */
public class DefaultIdentityProviderTestCase {

    private Subject subject;

    @Mock
    private Principal principalOne;

    @Mock
    private Principal principalTwo;

    @Mock
    private Group groupOne;

    @Mock
    private Group groupTwo;

    @BeforeMethod
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
        subject = new Subject();
    }

    @Test(expectedExceptions = IllegalStateException.class)
    public void testWithNoPrincipals() throws Exception {
        DefaultIdentityProvider provider = new DefaultIdentityProvider();
        provider.get(subject);
    }

    @Test(expectedExceptions = IllegalStateException.class)
    public void testWithMoreThanOneMonoValuedPrincipal() throws Exception {
        DefaultIdentityProvider provider = new DefaultIdentityProvider();
        subject.getPrincipals().add(principalOne);
        subject.getPrincipals().add(principalTwo);
        provider.get(subject);
    }

    @Test
    public void testWithOnlyOneMonoValuedPrincipal() throws Exception {
        when(principalOne.getName()).thenReturn("Guillaume");
        DefaultIdentityProvider provider = new DefaultIdentityProvider();
        subject.getPrincipals().add(principalOne);
        assertEquals(provider.get(subject), "Guillaume");
    }

    @Test
    public void testWithMixedPrincipals() throws Exception {
        when(principalOne.getName()).thenReturn("Guillaume");
        DefaultIdentityProvider provider = new DefaultIdentityProvider();
        subject.getPrincipals().add(principalOne);
        subject.getPrincipals().add(groupOne);
        subject.getPrincipals().add(groupTwo);
        assertEquals(provider.get(subject), "Guillaume");
    }
}
