/**
 * Copyright 2012-2013 Bull S.A.S.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.prompt;

import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.ow2.shelbie.core.prompt.Variables;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

/**
 * A {@code DefaultPromptProviderTestCase} is ...
 *
 * @author Guillaume Sauthier
 */
public class DefaultPromptProviderTestCase {

    @Mock
    private Variables variables;

    @BeforeMethod
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testPromptIsResolved() throws Exception {

        when(variables.get("user.name")).thenReturn("toto");
        when(variables.get("application.name")).thenReturn("shelbie");

        DefaultPromptService provider = new DefaultPromptService();
        assertEquals(provider.getPrompt(variables), "toto@shelbie$ ");
    }

    @Test
    public void testPromptCannotBeResolved() throws Exception {

        when(variables.get("user.name")).thenThrow(new RuntimeException());

        DefaultPromptService provider = new DefaultPromptService();
        assertEquals(provider.getPrompt(variables), "[shelbie]$ ");
    }
}
