/**
 * Copyright 2012 Bull S.A.S.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.it;


import org.apache.felix.service.command.CommandProcessor;
import org.apache.felix.service.command.CommandSession;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.ops4j.pax.exam.Option;
import org.ops4j.pax.exam.TestProbeBuilder;
import org.ops4j.pax.exam.junit.Configuration;
import org.ops4j.pax.exam.junit.ExamReactorStrategy;
import org.ops4j.pax.exam.junit.JUnit4TestRunner;
import org.ops4j.pax.exam.junit.ProbeBuilder;
import org.ops4j.pax.exam.spi.reactors.EagerSingleStagedReactorFactory;
import org.osgi.framework.Constants;

import javax.inject.Inject;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import static org.ops4j.pax.exam.CoreOptions.mavenBundle;
import static org.ops4j.pax.exam.CoreOptions.options;
import static org.ops4j.pax.exam.CoreOptions.provision;
import static org.ops4j.pax.exam.CoreOptions.systemProperty;

/**
 * A {@code GogoShellCommandsTestCase} is ...
 *
 * @author Guillaume Sauthier
 */
@RunWith(JUnit4TestRunner.class)
@ExamReactorStrategy(EagerSingleStagedReactorFactory.class)
public class GogoShellCommandsTestCase {

    @Inject
    private CommandProcessor processor;

    @Configuration
    public static Option[] config() {
        return options(
                provision(
                        // 13
                        // Gogo Bundles
                        mavenBundle("org.apache.felix", "org.apache.felix.gogo.runtime").versionAsInProject(),
                        mavenBundle("org.apache.felix", "org.apache.felix.gogo.command").versionAsInProject(),

                        // iPOJO Bundles
                        mavenBundle("org.apache.felix", "org.apache.felix.ipojo").versionAsInProject(),
                        mavenBundle("org.apache.felix", "org.apache.felix.ipojo.handler.whiteboard").versionAsInProject(),
                        //mavenBundle("org.apache.felix", "org.apache.felix.ipojo.arch.gogo").versionAsInProject(),

                        // Shelbie Bundles
                        // Intentionally inverted command and core bundles (See SHB-14)
                        mavenBundle("org.ow2.shelbie.commands", "shelbie-builtin-commands").versionAsInProject(),
                        mavenBundle("org.ow2.shelbie", "shelbie-core").versionAsInProject()
                ),

                // Force iPOJO to process metadata.xml synchronously otherwise PaxExam stops too early
                systemProperty("ipojo.processing.synchronous").value("true")
        );
    }

    @ProbeBuilder
    public TestProbeBuilder probeConfiguration(TestProbeBuilder probe) {
        // Ensure the Probe Bundle has the special package attribute
        probe.setHeader(Constants.DYNAMICIMPORT_PACKAGE, "org.apache.felix.service.*;status=provisional,*");
        return probe;
    }

    @Test
    public void testEchoCommandIsAvailable() throws Exception {
        executeCommand("shelbie:echo 'hello'");
    }

    @Test
    public void testGoshCommandIsAvailable() throws Exception {
        executeCommand("gosh --login --nointeractive --noshutdown");
    }

    private void executeCommand(String command) throws Exception {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PrintStream stream = new PrintStream(baos);

        CommandSession session = processor.createSession(System.in, stream, System.err);
        session.execute(command);

        System.out.println();
        System.out.println(">$ " + command);
        System.out.print(new String(baos.toByteArray()));
        System.out.println(" ------------------------------------------------------ ");

        session.close();
    }

}
